# 📖 Documentation

*This library and documentation is a work in progress. I'm always interested in receiving inspiration, suggestions, enhancements, literature and more. Feel free to file an [issue here](https://github.com/tmhglnd/total-serialism/issues) or make a pull request and I will gladly look into it!*

- [Generative Methods](./generative-methods.md)
	- [spread / spreadFloat](./generative-methods.md#spread-spreadfloat)
	- [spreadInclusive / spreadInclusiveFloat](./generative-methods.md#spreadinclusive-spreadinclusivefloat)
	- [spreadExp / spreadExpFloat](./generative-methods.md#spreadexp-spreadexpfloat)
	- [spreadInclusiveExp / spreadInclusiveExpFloat](./generative-methods.md#spreadinclusiveexp-spreadinclusiveexpfloat)
	- [fill](./generative-methods.md#fill)
	- [sine / cosine](./generative-methods.md#sine--cosine)
	- [sineFloat / cosineFloat](./generative-methods.md#sinefloat--cosinefloat)
	- [saw / sawFloat](./generative-methods.md#saw--sawfloat)
	- [square / squareFloat](./generative-methods.md#square--squarefloat)
	- [binary](./generative-methods.md#binarybeat)
	- [space](./generative-methods.md#spacingbeat)
- [Algorithmic Methods](./algorithmic-methods.md)
	- [euclid](./algorithmic-methods.md#euclidean)
	- [fastEuclid](./algorithmic-methods.md#fasteuclidean)
	- [hexBeat](./algorithmic-methods.md#hexbeat)
	- [linden](./algorithmic-methods.md#linden)
	- [Automaton](./algorithmic-methods.md#cellular-automaton)
	- [fibonacci](./algorithmic-methods.md#fibonacci)
	- [pisano](./algorithmic-methods.md#pisano)
	- [pell](./algorithmic-methods.md#pell)
	- [threeFibonacci](./algorithmic-methods.md#threefibonacci)
	- [lucas](./algorithmic-methods.md#lucas)
	- [nbonacci](./algorithmic-methods.md#nbonacci)
	- [infinitySeries](./algorithmic-methods.md#infinityseries)
- [Stochastic Methods](./stochastic-methods.md)
	- [seed](./stochastic-methods.md#seed)
	- [getSeed](./stochastic-methods.md#getseed)
	- [random](./stochastic-methods.md#random-randomfloat)
	- [drunk](./stochastic-methods.md#drunk-drunkfloat)
	- [coin](./stochastic-methods.md#coin)
	- [dice](./stochastic-methods.md#dice)
	- [urn](./stochastic-methods.md#urn)
	- [twelveTone](./stochastic-methods.md#twelvetone)
	- [shuffle](./stochastic-methods.md#shuffle)
	- [choose](./stochastic-methods.md#choose)
	- [pick](./stochastic-methods.md#pick)
	- [clave](./stochastic-methods.md#clave)
	- [expand](./stochastic-methods.md#expand)
	- [MarkovChain](./stochastic-methods.md#markovchain)
	- [DeepMarkovChain](./stochastic-methods.md#deepmarkovchain)
- [Transform Methods](./transform-methods.md)
	- [clone](./transform-methods.md#clone)
	- [join](./transform-methods.md#join)
	- [copy](./transform-methods.md#copy)
	- [every](./transform-methods.md#every)
	- [flatten](./transform-methods.md#flatten)
	- [filter](./transform-methods.md#filter)
	- [filterType](./transform-methods.md#filterType)
	- [invert](./transform-methods.md#invert)
	- [lace](./transform-methods.md#lace)
	- [lookup](./transform-methods.md#lookup)
	- [merge](./transform-methods.md#merge)
	- [palindrome](./transform-methods.md#palindrome)
	- [repeat](./transform-methods.md#repeat)
	- [reverse](./transform-methods.md#reverse)
	- [rotate](./transform-methods.md#rotate)
	- [sort](./transform-methods.md#sort)
	- [spray](./transform-methods.md#spray)
	- [slice](./transform-methods.md#slice)
	- [split](./transform-methods.md#split)
	- [stretch](./transform-methods.md#stretch)
	- [unique](./transform-methods.md#unique)
- [Statistic Methods](./statistic-methods.md)
	- [sort](./statistic-methods.md#sort)
	- [average](./statistic-methods.md#average) (mean)
	- [center](./statistic-methods.md#center) (median)
	- [common](./statistic-methods.md#common) (mode)
	- [maximum](./statistic-methods.md#maximum)
	- [minimum](./statistic-methods.md#minimum)
	- [change](./statistic-methods.md#change)
	- [compare](./statistic-methods.md#compare)
- [Translate Methods](./translate-methods.md)
	- [Conversion between pitch units](./translate-methods.md#conversion-between-pitch-units)
		- midiToNote (mton)
		- midiToFreq (mtof)
		- noteToMidi (ntom)
		- noteToFreq (ntof)
		- freqToMidi (ftom)
		- freqToNote (fton)
		- relativeToMidi (rtom)
		- relativeToFreq (rtof)
		- ratioToCent (rtoc)
	- [Conversion between time units](./translate-methods.md#conversion-between-time-units)
		- divisionToMs (dtoms)
		- divisionToRatio (dtor)
		- divisionToTicks (dtotk)
		- timevalueToRatio (ttor)
		- timevalueToTicks (ttotk)
	- [Scala Tuning](./translate-methods.md#using-scala-tuning-format)
		- Scala.parse
		- Scala.tune
		- Scala.center
		- Scala.scalaToFreq
		- Scala.chart
		- Scala.data
		- Scala.names
		- Scala.search
	- [Working with predefined scale and root](./translate-methods.md#working-with-fixed-scale-and-root)
		- setTempo
		- setScale
		- setRoot
		- getSettings
		- scaleNames
		- relativeToScale (toScale)
		- relativeToMidi (toMidi)
		- relativeToFreq (toFreq)
- [Utility Methods](./utility-methods.md)
	- [wrap](./utility-methods.md#wrap)
	- [constrain](./utility-methods.md#constrain)
	- [fold](./utility-methods.md#fold)
	- [scale](./utility-methods.md#scale)
	- [add](./utility-methods.md#add)
	- [subtract](./utility-methods.md#subtract)
	- [multiply](./utility-methods.md#multiply)
	- [divide](./utility-methods.md#divide)
	- [mod](./utility-methods.md#mod)
	- [pow](./utility-methods.md#pow)
	- [sqrt](./utility-methods.md#sqrt)
	- [arrayCalc](./utility-methods.md#arraycalc)
	- [sum](./utility-methods.md#sum)
	- [minimum](./utility-methods.md#minimum)
	- [maximum](./utility-methods.md#maximum)
	- [normalize](./utility-methods.md#normalize)
	- [flatten](./utility-methods.md#flatten)
	- [plot](./utility-methods.md#plot)
	- [draw](./utility-methods.md#draw)